/* 
 *   Creation Date: <2001/10/29 00:29:21 samuel>
 *   Time-stamp: <2003/06/24 14:02:18 samuel>
 *   
 *	<byteorder.h>
 *	
 *	Big endian/Little endian conversion
 *   
 *   Copyright (C) 2001, 2003 Samuel Rydh (samuel@ibrium.se)
 *
 *   The swap macros are borrowed from the the swap.h linux header.
 *   
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation
 *   
 */

#ifndef _H_BYTEORDER
#define _H_BYTEORDER

#include <byteswap.h>

#ifdef __linux__
#include <endian.h>
#else
#include <machine/endian.h>
#endif

#define	le32_to_cpu( x )	cpu_to_le32((x))
#define	le16_to_cpu( x )	cpu_to_le16((x))

#ifdef __ppc__
static __inline__ unsigned ld_le16( const unsigned short *addr) {
        unsigned val;
        __asm__ ("lhbrx %0,0,%1" : "=r" (val) : "r" (addr), "m" (*addr) );
        return val;
}
static __inline__ void st_le16( unsigned short *addr, unsigned short val) {
        __asm__ ("sthbrx %1,0,%2" : "=m" (*addr) : "r" (val), "r" (addr) );
}
static __inline__ unsigned ld_le32(const ulong *addr) {
        unsigned val;
        __asm__ ("lwbrx %0,0,%1" : "=r" (val) : "r" (addr), "m" (*addr) );
        return val;
}
static __inline__ void st_le32(ulong *addr, ulong val) {
        __asm__ ("stwbrx %1,0,%2" : "=m" (*addr) : "r" (val), "r" (addr) );
}
#define cpu_to_le32( x ) \
({ \
        u32 __x = (x); \
        ((u32)( \
                (((u32)(__x) & (u32)0x000000ffUL) << 24) | \
                (((u32)(__x) & (u32)0x0000ff00UL) <<  8) | \
                (((u32)(__x) & (u32)0x00ff0000UL) >>  8) | \
                (((u32)(__x) & (u32)0xff000000UL) >> 24) )); \
})
#define cpu_to_le16(x) \
({ \
        u16 __x = (x); \
        ((u16)( \
                (((u16)(__x) & (u16)0x00ffU) << 8) | \
                (((u16)(__x) & (u16)0xff00U) >> 8) )); \
})
#endif

#ifdef __i386__
#include <byteswap.h>
static __inline__ unsigned	ld_be16( const unsigned short *addr) { return bswap_16( *addr ); }
static __inline__ void		st_be16( unsigned short *addr, unsigned short val) { *addr = bswap_16(val); }
static __inline__ unsigned	ld_be32(const ulong *addr) { return bswap_32( *addr ); }
static __inline__ void		st_be32(ulong *addr, ulong val) { *addr = bswap_32(val); }
#endif

#if BYTE_ORDER == BIG_ENDIAN

#define BE32_FIELD( dummy )	do {} while(0)
#define BE16_FIELD( dummy )	do {} while(0)

static __inline__ unsigned	ld_be16( const unsigned short *addr) { return *addr; }
static __inline__ void		st_be16( unsigned short *addr, unsigned short val) { *addr = val; }
static __inline__ unsigned	ld_be32( const ulong *addr) { return *addr; }
static __inline__ void		st_be32(ulong *addr, ulong val) { *addr = val; }

#else /* BYTE_ORDER == BIG_ENDIAN */

/* careful - these macros are not typechecked */
#define BE32_FIELD( field )	do { field = ld_be32( (ulong*)&field ); } while (0)
#define BE16_FIELD( field )	do { field = ld_be16( (ushort*)&field ); } while (0)

static __inline__ unsigned	ld_le16( const unsigned short *addr) { return *addr; }
static __inline__ void		st_le16( unsigned short *addr, unsigned short val) { *addr = val; }
static __inline__ unsigned	ld_le32(const ulong *addr) { return *addr; }
static __inline__ void		st_le32(ulong *addr, ulong val) { *addr = val; }

#endif /* BYTE_ORDER == BIG_ENDIAN */

#endif   /* _H_BYTEORDER */
